// Compatiblity layer with ibex for the simple interval library from Luc JAULIN, with minor modifications from Fabrice LE BARS and Jeremy NICOLA.

#include "box.h"

using namespace std;

//----------------------------------------------------------------------
// Constructors/destructors
//----------------------------------------------------------------------
box::box()
{
	dim = 0;
	data = new interval[1];
}
//----------------------------------------------------------------------
box::box(int a)
{
	dim = a;
	data = new interval[dim];
}
//----------------------------------------------------------------------
box::box(interval x)
{
	dim = 1;
	data = new interval[dim];
	(*this)[1] = x;
}
//----------------------------------------------------------------------
box::box(interval x, interval y)
{
	dim = 2;
	data = new interval[dim];
	(*this)[1] = x;
	(*this)[2] = y;
}
//----------------------------------------------------------------------
box::box(interval x, interval y, interval z)
{
	dim = 3;
	data = new interval[dim];
	(*this)[1] = x;
	(*this)[2] = y;
	(*this)[3] = z;
}
//----------------------------------------------------------------------
box::box(interval x, int n)
{
	dim = n;
	data = new interval[dim];
	for (int k = 1; k <= dim; k++) (*this)[k] = x;
}
//----------------------------------------------------------------------
box::box(const box& X)
{
	if (&X == this) return;
	dim = Size(X);
	data = new interval[dim];
	for (int k = 1; k <= dim; k++) (*this)[k] = X[k];
}
//----------------------------------------------------------------------
box::~box() { delete[] data; }

//----------------------------------------------------------------------
// Operators
//----------------------------------------------------------------------

box& box::operator=(const box& X)
{
	delete[] data;
	dim = Size(X);
	data = new interval[dim];
	for (int k = 1; k <= dim; k++) (*this)[k] = X[k];
	return *this;
}
//----------------------------------------------------------------------
box operator+(const box& X, const box& Y)
{
	box Ans(Size(X));
        for (int k = 1; k <= Size(Ans); k++)
        { interval I=X[k]+Y[k];
          Ans[k] =I ;}
	return Ans;
}
//----------------------------------------------------------------------
box operator-(const box& X)
{
	box Ans(Size(X));
        for (int k = 1; k <= Size(Ans); k++)
        {
            interval I= -X[k];
            Ans[k] =I;
        }
	return Ans;
}
//----------------------------------------------------------------------
box operator-(const box& X, const box& Y)
{
	box Ans(Size(X));
	for (int k = 1; k <= Size(Ans); k++) Ans[k] = X[k] - Y[k];
	return Ans;
}
//----------------------------------------------------------------------
box operator*(const interval& a, const box& X)
{
	box Ans(Size(X));
	for (int k = 1; k <= Size(Ans); k++) Ans[k] = a*X[k];
	return Ans;
}
//----------------------------------------------------------------------
box operator*(const box& X, const interval& a)
{
	box Ans(Size(X));
	for (int k = 1; k <= Size(Ans); k++) Ans[k] = a*X[k];
	return Ans;
}
//----------------------------------------------------------------------
box operator*(const double a, const box& X)
{
	box Ans(Size(X));
	for (int k = 1; k <= Size(Ans); k++) Ans[k] = a*X[k];
	return Ans;
}
//----------------------------------------------------------------------
box operator*(const box& X, const double a)
{
	box Ans(Size(X));
	for (int k = 1; k <= Size(Ans); k++) Ans[k] = a*X[k];
	return Ans;
}
//----------------------------------------------------------------------
box operator&(const box& X, const box& Y)
{
	return Inter(X, Y);
}
//----------------------------------------------------------------------
box operator|(const box& X, const box& Y)
{  
	return Union(X, Y);
}
//----------------------------------------------------------------------

interval& box::operator[](int i) const
{
	return data[i - 1];
}
//----------------------------------------------------------------------
bool box::IsEmpty(void) const
{
	if (dim == 0) return true;
	for (int i = 1; i <= dim; i++)
	{
                if ((*this)[i].isEmpty) return true;
	}
	return false;
}

//*********************************************************************
//******************    FONCTION DE DESSIN     **********************
//*********************************************************************
/*
//----------------------------------------------------------------------
void box::DrawBox(const char* param)
{vibes::drawBox((*this)[1].inf,(*this)[1].sup,(*this)[2].inf,(*this)[2].sup,param);
}
//----------------------------------------------------------------------
void DrawArrow(double x1,double y1,double dx,double dy, double r, const char* param)
{       vector<double> X,Y;
        double x2=x1+dx;
        double y2=y1+dy;
        X.push_back(x1);    Y.push_back(y1);
        X.push_back(x2); Y.push_back(y2);
        double a=3.14-(3.14/4);
        double px=x2+r*(cos(a)*dx-sin(a)*dy);    // cot de pointe
        double py=y2+r*(sin(a)*dx+cos(a)*dy);
        double qx=x2+r*(cos(-a)*dx-sin(-a)*dy);  // autre cot de pointe
        double qy=y2+r*(sin(-a)*dx+cos(-a)*dy);
        X.push_back(px); Y.push_back(py);
        X.push_back(x2); Y.push_back(y2);
        X.push_back(qx); Y.push_back(qy);
        vibes::drawLine(X,Y,param);
 }
//----------------------------------------------------------------------
void DrawPolygon(double x,double y,double theta,vector<double> X, vector<double> Y,const char* param)
{       vector<double> X1,Y1;
        for (int k=0;k<X.size();k++)
        {   double x1=x+cos(theta)*X[k]-sin(theta)*Y[k];
            double y1=y+sin(theta)*X[k]+cos(theta)*Y[k];
            X1.push_back(x1); Y1.push_back(y1);
        }
        vibes::drawLine(X1,Y1,param);
}
//--------------------------------------------------------------------------------------------------
void DrawRobot(double x,double y,double theta, double s,const char* param)
{   vector<double> X,Y;
    X.push_back(0*s); Y.push_back(-1*s);
    X.push_back(3*s); Y.push_back(0*s);
    X.push_back(0*s); Y.push_back(1*s);
    X.push_back(0*s); Y.push_back(-1*s);
    DrawPolygon(x,y,theta,X,Y,param);
}
//--------------------------------------------------------------------------------------------------
void DrawSquare(double x,double y,double theta, double s,const char* param)
{   vector<double> X,Y;
    X.push_back(-s); Y.push_back(s);
    X.push_back(s); Y.push_back(s);
    X.push_back(s); Y.push_back(-s);
    X.push_back(-s); Y.push_back(-s);
    X.push_back(-s); Y.push_back(s);
    DrawPolygon(x,y,theta,X,Y,param);
}*/

//*********************************************************************
//******************    FONCTION NON MEMBRES     **********************
//*********************************************************************
//----------------------------------------------------------------------



//----------------------------------------------------------------------
box EmptyBox(int n)
    {return (box(n)); }
//----------------------------------------------------------------------
box EmptyBox(const box& X)
    { return (box(Size(X)));}

//----------------------------------------------------------------------
box Inter(const box& X, const box& Y)
{	box Ans(Size(X));
	if ((X.IsEmpty()) || (Y.IsEmpty())) { return Ans; }
	for (int k = 1; k <= Size(Ans); k++)
        {       Ans[k] = Inter(X[k], Y[k]);
                if (Ans[k].isEmpty) { Update(Ans); return Ans; }
	}
	return Ans;
}
//----------------------------------------------------------------------
box Union(const box& X, const box& Y)
{	box Ans(Size(X));
	if (X.IsEmpty()) return (Y);
	if (Y.IsEmpty()) return (X);
	for (int k = 1; k <= Size(Ans); k++)  Ans[k] = Union(X[k], Y[k]);
	return Ans;
}

//----------------------------------------------------------------------
// Other functions
//----------------------------------------------------------------------
int Size(const box& X) { return (X.dim); }
//----------------------------------------------------------------------
double Width(const box& X)
{
	return Width(X[AxePrincipal(X)]);
}

//----------------------------------------------------------------------
void Update(const box& X)
{ for (int i = 1; i <= Size(X); i++)
       if (X[i].isEmpty)
        {	for (int j = 1; j <= Size(X); j++) X[j] = interval();
                return;
        }
}
//----------------------------------------------------------------------
int AxePrincipal(const box& X)
{	int kmax = 1;
	double widthmax = Width(X[kmax]);
	for (int k = 2; k <= Size(X); k++)
        if (Width(X[k]) > widthmax)
                {kmax = k; widthmax = Width(X[k]);}
        return kmax;
}

//----------------------------------------------------------------------
// Contractors
//----------------------------------------------------------------------
void Cadd(box& Z, box& X, box& Y)
{
	for (int k = 1; k <= Size(X); k++)
                Cadd(Z[k], X[k], Y[k]);
}
//----------------------------------------------------------------------
void Csub(box& Z, box& X, box& Y)
{
	for (int k = 1; k <= Size(X); k++)
                Csub(Z[k], X[k], Y[k]);
}

//----------------------------------------------------------------------
void Cnorm(interval& R, box& X)
{	int imax = X.dim;
	box Sum2(imax);
	box X2(imax);
	for (int i = 1; i <= imax; i++)
        {       X2[i] = interval(-oo, oo);
                Csqr(X2[i], X[i]);
	}
	Sum2[1] = X2[1];
	for (int i = 2; i <= imax; i++)
	{
		Sum2[i] = interval(-oo, oo);
                Cadd(Sum2[i], Sum2[i - 1], X2[i]);
	}
        Csqr(Sum2[imax], R);
        Csqr(Sum2[imax], R);
	for (int i = imax; i >= 2; i--)
	{
                Cadd(Sum2[i], Sum2[i - 1], X2[i]);
	}
	X2[1] = Sum2[1];
	for (int i = imax; i >= 1; i--)
                Csqr(X2[i], X[i]);
}
//----------------------------------------------------------------------
void Cdist(interval& R, box& X, box& Y)
{	box Z = Y - X;
	Cnorm(R, Z);
        Csub(Z, Y, X);
}

//----------------------------------------------------------------------
void Bisect(box& X, box& X1, box& X2)
{
	BisectAlong(X, X1, X2, AxePrincipal(X));
}
//----------------------------------------------------------------------
void BisectAlong(box& X, box& X1, box& X2, int i)
{
	X1 = X2 = X;
        double m = ((X[i].inf)+(1.01*X[i].sup))/2.01;
        X1[i] = interval(X1[i].inf,m);
        X2[i] = interval(m,X2[i].sup);
}

