// Compatiblity layer with ibex for the simple interval library from Luc JAULIN, with minor modifications from Fabrice LE BARS and Jeremy NICOLA.

#include "interval.h"
#include <float.h>
#include <cmath>

using namespace std;

const double PI = M_PI;
const double PI_2 = M_PI_2;
const double TROIS_PI_2 = 3*PI_2;
const double CINQ_PI_2 = 5*PI_2;
const double SEPT_PI_2 = 7*PI_2;
const double DEUX_PI = 4*PI_2;

//**********************************************************************
// Intervalle
//**********************************************************************

interval::interval ()
{   inf=-999;  sup=-999;
    isEmpty=true;
}
//----------------------------------------------------------------------
interval::interval (const double& m)
{   inf=m;  sup=m;
    isEmpty=false;
}
//----------------------------------------------------------------------
interval::interval (const double& a, const double& b)
{   if  (a>=b)   {inf = b ; sup = a;} //modif sur >=
    else        {inf = a ; sup = b;}
    isEmpty=false;
}
interval::interval (const interval& a) {*this = a;}

//**********************************************************************
// Operateurs
//**********************************************************************

//----------------------------------------------------------------------
interval& interval::operator= (const interval& v)
{(*this).inf=v.inf; (*this).sup=v.sup; (*this).isEmpty=v.isEmpty;
  return *this;}
//----------------------------------------------------------------------
interval operator& (const interval& x,const interval& y)
{ return (Inter(x,y));}
//----------------------------------------------------------------------
interval operator+ (const interval& x,const interval& y)
{ if (x.isEmpty||y.isEmpty) return interval();
  else  return (interval(x.inf+y.inf,x.sup+y.sup));}
//----------------------------------------------------------------------
interval operator- (const interval& a)
{ if (a.isEmpty) return interval();
  else return(interval(-(a.sup),-(a.inf))); }
//----------------------------------------------------------------------
interval operator- (const interval& x, const interval& y)
{ if (x.isEmpty||y.isEmpty) return interval();
  else return(x+(-y));}
//----------------------------------------------------------------------
interval operator* (const double x, const interval& y)
{return interval(x*y.inf,x*y.sup); }
//----------------------------------------------------------------------
interval operator* (const interval x, const double& y)
{return interval(y*x); }
//----------------------------------------------------------------------
interval operator* (const interval& x, const interval& y)
{
if (x.isEmpty||y.isEmpty) return interval();
double x1=x.inf,y1=y.inf,x2=x.sup,y2=y.sup;
return interval(min(min(x1*y1,x1*y2),min(x2*y1,x2*y2)),
                max(max(x1*y1,x1*y2),max(x2*y1,x2*y2)));
}
//----------------------------------------------------------------------
interval operator/  (const interval& a, const interval& b)
{
if (a.isEmpty||b.isEmpty) return interval();
if (b.inf> 0)
    {
    if    (a.inf>=0) return interval(a.inf/b.sup, a.sup/b.inf);
    else if (a.sup<0)  return interval(a.inf/b.inf, a.sup/b.sup);
           else  return interval(a.inf/b.inf, a.sup/b.inf);
    }
    else if (b.sup<0)
          {
          if (a.inf>=0) return interval(a.sup/b.sup, a.inf/b.inf);
            else if (a.sup<0)  return interval(a.sup/b.inf, a.inf/b.sup);
                  else return interval(a.sup/b.sup, a.inf/b.sup);
          }
          else return interval(-oo,oo);
}

//**********************************************************************
// Operations
//**********************************************************************
double Width(const interval& a)
{
if (a.isEmpty) return (-1);
else return (a.sup-a.inf);
}
//----------------------------------------------------------------------
interval Inter(const interval& a, const interval& b)
{
if (a.isEmpty || b.isEmpty) return interval();
interval r(0,0);
if ((a.inf>b.sup)||(b.inf>a.sup)) return interval();
if (a.inf<=b.inf) r.inf=b.inf; else r.inf=a.inf;
if (a.sup>=b.sup) r.sup=b.sup; else r.sup=a.sup;
return r;
}
//----------------------------------------------------------------------
interval Union(const interval& a, const interval& b)
{
    interval b1;
    b1=b;
if (a.isEmpty) return b1;
if (b.isEmpty) return a;
interval r(0,0);
if (a.inf<=b.inf) r.inf=a.inf; else r.inf=b.inf;
if (a.sup>=b.sup) r.sup=a.sup; else r.sup=b.sup;
return r;
}
//-----------------------------------------------------------------------
double Center(const interval& a)
{
 if (a.isEmpty) return oo;
 else  return (a.sup+a.inf)/2;
}
//----------------------------------------------------------------------
interval Sqr(const interval& a)
{   double a1=a.inf, a2=a.sup;
    if (a.isEmpty) return interval();
    if ((a1>=0)||(a2<=0))   return interval(a1*a1,a2*a2);
    if (fabs(a1)>fabs(a2))  return interval(0,a1*a1);
    else                    return interval(0,a2*a2);
}
//----------------------------------------------------------------------
interval Sqrt(const interval& a)
{   if ((a.isEmpty)||(a.sup<0)) return interval();
    double a1=a.inf, a2=a.sup;
    if (a1>=0)  return (interval(sqrt(a1),sqrt(a2)));
    else return (interval(0,sqrt(a2)));
}
//----------------------------------------------------------------------
interval Modulo(const interval& a, double x)
{   if ((a.inf>=0)&&(a.inf<x)) return (a);
    int k = (long)floorl((a.inf/x));
    double offset = x * k;
    return interval(a.inf-offset,a.sup-offset);
}
//----------------------------------------------------------------------
double Rand(const interval& a)
{
 if (a.isEmpty) return oo;
 else  return a.inf+(a.sup-a.inf)*rand()/RAND_MAX;
}
//----------------------------------------------------------------------
interval Sin (const interval& a)
{
    if (a.isEmpty) return interval();
    if (a.sup==a.inf) return sin(a.inf);
    interval b;
    double sin1,sin2,r1,r2;
    b = Modulo(a,DEUX_PI);
    if (Width(a)>DEUX_PI) return (interval(-1,1));
    sin1=sin(b.inf);   sin2=sin(b.sup);
    if ((b.inf < TROIS_PI_2)&&(b.sup > TROIS_PI_2)) r1=-1.0;
    else if ((b.inf < SEPT_PI_2)&&(b.sup > SEPT_PI_2)) r1=-1.0;
    else r1=((sin1 < sin2)? sin1 : sin2);
    if ((b.inf < PI_2)&&(b.sup > PI_2)) r2=1.0;
    else if ((b.inf < CINQ_PI_2)&&(b.sup > CINQ_PI_2)) r2=1.0;
    else r2=((sin1 > sin2)? sin1 : sin2);
    return Inter(interval(-1,1),interval(r1,r2));
}
//----------------------------------------------------------------------
interval Cos(const interval& a)
{   if (a.isEmpty) return interval();
    else return Sin(a+PI_2);
}

//**********************************************************************
// Contractors
//**********************************************************************

//----------------------------------------------------------------------
void Cadd(interval& Z, interval& Y, interval& X)
{ Z=Inter(Z,Y+X);
  Y=Inter(Y,Z-X); X=Inter(X,Z-Y);
}
//----------------------------------------------------------------------
void Cadd(interval& Z, double& X, interval& Y)
{  Z=Inter(Z,X+Y);
   Y=Inter(Y,Z-X);
}
//----------------------------------------------------------------------
void Cadd(double& Z, interval& Y, interval& X)
{
  Y=Inter(Y,Z-X); X=Inter(X,Z-Y);
}
//----------------------------------------------------------------------
void Csub(interval& Z, interval& Y, interval& X)
{ Z=Inter(Z,Y-X);
  Y=Inter(Y,Z+X); X=Inter(X,Y-Z);
}
//----------------------------------------------------------------------
void Csub(double& Z, interval& Y, interval& X)
{ Y=Inter(Y,Z+X); X=Inter(X,Y-Z);
}
//----------------------------------------------------------------------
void Csub(interval& Z, interval& Y, double& x)
{ Z=Inter(Z,Y-x);
  Y=Inter(Y,Z+x);
}
//----------------------------------------------------------------------
void Csub(interval& Z, double& y, interval& X)
{ Z=Inter(Z,y-X);
  X=Inter(X,y-Z);
}
//----------------------------------------------------------------------
void Csqr(interval& Y, interval& X)

{  interval Yd, Yg, Xd=Inter(X, interval(0,oo)), Xg=Inter(X,interval(-oo,0));
   if (!Xd.isEmpty) {Xd=Inter(Xd,Sqrt(Y));  Yd=Inter(Y,Sqr(Xd));}
   if (!Xg.isEmpty) {Xg=Inter(Xg,-Sqrt(Y)); Yg=Inter(Y,Sqr(Xg));}
   Y=Inter(Y,Union(Yd,Yg));
   X=Inter(X,Union(Xd,Xg));
}
//----------------------------------------------------------------------
void Csqrt(interval& Y, interval& X)

{ Y=Inter(Y,Sqrt(X));
  Csqr(X,Y);
}
//----------------------------------------------------------------------
void CinRing(interval& mx, interval& my, double cx, double cy, interval R)
{      // contracte relativement  la contrainte : "m appartient au cercle de centre c et de rayon r"
       interval d_x=mx-cx;
       interval d_y=my-cy;
       interval d2_x=Sqr(d_x);
       interval d2_y=Sqr(d_y);
       interval r2=Sqr(R);
       Cadd(r2,d2_x,d2_y);
       Csqr(d2_x,d_x);
       Csqr(d2_y,d_y);
       mx=Inter(mx,d_x+cx);
       my=Inter(my,d_y+cy);
       if ((mx.isEmpty)||(my.isEmpty))  {mx.isEmpty=true; my.isEmpty=true;};
}
//----------------------------------------------------------------------

